using System;
using System.ComponentModel;
using System.Drawing;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;
using System.Resources;
using System.Reflection;
using System.Globalization;

namespace ReflectionIT.Windows.Forms {

	/// <summary>
	/// The LocalizedTextProvider makes it easy to set the localized text for a control. 
	/// 
	/// It extends contols with the extra LocalizedText property
	/// </summary>
	/// <remarks>
	/// Copyright  2003 Reflection IT
	/// 
	/// This software is provided 'as-is', without any express or implied warranty.
	/// In no event will the authors be held liable for any damages arising from the
	/// use of this software.
	/// 
	/// Permission is granted to anyone to use this software for any purpose,
	/// including commercial applications, subject to the following restrictions:
	/// 
	/// 1. The origin of this software must not be misrepresented; you must not claim
	/// that you wrote the original software. 
	/// 
	/// 2. No substantial portion of the source code of this library may be redistributed
	/// without the express written permission of the copyright holders, where
	/// "substantial" is defined as enough code to be recognizably from this library. 
	/// </remarks>
	[ProvideProperty("LocalizedText", typeof(Component))]
	public class LocalizedTextProvider : System.ComponentModel.Component, System.ComponentModel.IExtenderProvider, ISupportInitialize {
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private Hashtable _localizedTextList = new Hashtable();
		internal const string ResourceKey = "ReflectionIT.Windows.Forms.Resources.Microsoft";
		internal readonly ResourceManager ResourceManager = new ResourceManager( ResourceKey, System.Reflection.Assembly.GetExecutingAssembly());
		internal const string DefaultText = "(None)";

		/// <summary>
		/// Required for Windows.Forms Class Composition Designer support
		/// </summary>
		public LocalizedTextProvider(System.ComponentModel.IContainer container) {
			container.Add(this);
			InitializeComponent();
		}

		/// <summary>
		/// Required for Windows.Forms Class Composition Designer support
		/// </summary>
		public LocalizedTextProvider() {
			InitializeComponent();
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			components = new System.ComponentModel.Container();
		}
		#endregion

		/// <summary>
		/// Specifies whether this object can provide its extender properties to the specified object.
		/// </summary>
		/// <param name="target">The Object to receive the extender properties.</param>
		/// <returns>true if this object can provide extender properties to the specified object; otherwise, false</returns>
		bool IExtenderProvider.CanExtend(object target) {
			return (target is Control) | (target is MenuItem);
		}

		/// <summary>
		/// Get the key of the LocalizedText for the given component
		/// </summary>
		/// <param name="component">Control or MenuItem</param>
		/// <returns>Key of the LocalizedText</returns>
		[Category("Misc")]
		[Editor(typeof(LocalizedTextEditor), typeof(System.Drawing.Design.UITypeEditor))]
		[DefaultValue(LocalizedTextProvider.DefaultText)]
		[Description("The LocalizedText for this control")]
		public string GetLocalizedText(Component component) {
			if (_localizedTextList.ContainsKey(component)) {
				if (_localizedTextList[component] != null) {
					return (string)(_localizedTextList[component]);
				}
			}

			return LocalizedTextProvider.DefaultText;
		}
		
		/// <summary>
		/// Set the LocalizedText for the given component
		/// </summary>
		/// <param name="component">Control or MenuItem</param>
		/// <param name="value">Key of the LocalizedText</param>
		public void SetLocalizedText(Component component, string value) {
			_localizedTextList[component] = value;

			if (value != null && value != LocalizedTextProvider.DefaultText) {
				UpdateText(component);
			}		
		}

		/// <summary>
		/// Update the text for the component
		/// </summary>
		/// <param name="component">Control or MenuItem</param>
		private void UpdateText(Component component) {
			string s = GetLocalizedText(component);

			if (s != null && s.Length > 0) {
				if (component is Control) {
					((Control)component).Text = ResourceManager.GetString(s);
				} else {
					if (component is MenuItem) {
						((MenuItem)component).Text = ResourceManager.GetString(s);
					}
				}
			}
		}

		#region Implementation of ISupportInitialize

		public void BeginInit() {
			// ignore
		}

		/// <summary>
		/// Update the Text for the all components, overwrites the manually set Text
		/// </summary>
		public void EndInit() {
			foreach (Component component in _localizedTextList.Keys) {
				UpdateText(component);
			}
		}

		#endregion

	}

	/// <summary>
	/// TypeEditor for LocalizedText used by the LocalizedTextProvider
	/// </summary>
	public class LocalizedTextEditor : PeterBlum.UITypeEditorClasses.BaseDropDownListTypeEditor {

		private static ArrayList _keys;

		/// <summary>
		/// Static Constructor: Create a list of all Keys from the ResourceFile
		/// </summary>
		static LocalizedTextEditor() {
			_keys = new ArrayList();
			Assembly a = Assembly.GetExecutingAssembly();
			System.IO.Stream rs = a.GetManifestResourceStream(LocalizedTextProvider.ResourceKey + ".resources");
			if (rs != null) {
				ResourceReader rr = new ResourceReader(rs);

				foreach (DictionaryEntry d in rr) {
					if (d.Value is string) {
						_keys.Add((string)d.Key);
					}
				}
			}
		}

		/// <summary>
		/// Add all keys to the ListBox
		/// </summary>
		/// <param name="pContext"></param>
		/// <param name="pProvider"></param>
		/// <param name="pListBox"></param>
		protected override void FillInList(ITypeDescriptorContext pContext, IServiceProvider pProvider, ListBox pListBox) {
			pListBox.Items.Add(LocalizedTextProvider.DefaultText);
			foreach (string key in _keys) {
				pListBox.Items.Add(key);
			}
			pListBox.Sorted = true;
		}

	}  

}
